from typing import Generic, TypeVar

from rainbowadn.collection.comparison import *
from rainbowadn.core import *
from .binarycreation import *
from .binaryprotocolized import *

__all__ = ('BinaryBalancing',)

TreeType = TypeVar('TreeType')
ActiveKeyType = TypeVar('ActiveKeyType')
MetaDataType = TypeVar('MetaDataType')


class BinaryBalancing(Generic[ActiveKeyType, MetaDataType, TreeType]):
    def __init__(self, comparator_: Comparator[ActiveKeyType], empty_metadata: HashPoint[MetaDataType]):
        assert isinstance(comparator_, Comparator)
        assert isinstance(empty_metadata, HashPoint)
        self.comparator = comparator_
        self.empty_metadata = empty_metadata

    async def metadata(
            self,
            treel: TreeType,
            treer: TreeType,
            key: HashPoint[ActiveKeyType],
            creation: BinaryCreation[ActiveKeyType, MetaDataType, TreeType],
    ) -> HashPoint[MetaDataType]:
        raise NotImplementedError

    async def verify_metadata(
            self,
            treel: TreeType,
            treer: TreeType,
            key: HashPoint[ActiveKeyType],
            metadata: HashPoint[MetaDataType],
            creation: BinaryCreation[ActiveKeyType, MetaDataType, TreeType],
    ) -> bool:
        assert_eq(await self.metadata(treel, treer, key, creation), metadata)
        return True

    async def balance(
            self,
            protocolized: BinaryProtocolized[ActiveKeyType, MetaDataType, TreeType],
    ) -> TreeType:
        raise NotImplementedError
